@@ -395,11 +395,7 @@ export default class ClusterModel extends KubeModel {
     });
 
     // get resource end point for each resource
-    const k8sPaths = await this.kubeConnector.get('/').catch((err) => {
-      logger.error(err);
-      throw err;
-    });
-    const requestPaths = await Promise.all(resources.map(async (resource) => this.getResourceEndPoint(resource, k8sPaths)));
+    const requestPaths = await Promise.all(resources.map(async (resource) => this.getResourceEndPoint(resource)));
     if (requestPaths.length > 0) {
       const missingTypes = [];
       const missingEndPoints = [];
@@ -481,27 +477,22 @@ export default class ClusterModel extends KubeModel {
 
     // if the only errors were "already existing", patch those resources
     if (errors.length === 0 && updates.length > 0) {
-      // get the selfLinks of the existing resources
-      const existing = await Promise.all(updates.map(({ requestPath, resource }) => {
+      // Update the existing resources
+      const replaced = await Promise.all(updates.map(({ requestPath, resource }) => {
         const name = _.get(resource, 'metadata.name');
-        return this.kubeConnector.get(`${requestPath}/${name}`).catch((err) => {
-          logger.error(err);
-          throw err;
-        });
-      }));
-
-      // then update the resources
-      const replaced = await Promise.all(updates.map(({ resource }, index) => {
-        const selfLink = _.get(existing, `[${index}].metadata.selfLink`);
-        const resourceVersion = _.get(existing, `[${index}].metadata.resourceVersion`);
-        _.set(resource, 'metadata.resourceVersion', resourceVersion);
-        const requestBody = {
-          body: resource,
-        };
-        return this.kubeConnector.put(`${selfLink}`, requestBody).catch((err) => {
-          logger.error(err);
-          throw err;
-        });
+        const path = `${requestPath}/${name}`;
+        return this.kubeConnector.get(path)
+          .then((existing) => {
+            const resourceVersion = _.get(existing, 'metadata.resourceVersion');
+            _.set(resource, 'metadata.resourceVersion', resourceVersion);
+            const requestBody = {
+              body: resource,
+            };
+            return this.kubeConnector.put(path, requestBody);
+          }).catch((err) => {
+            logger.error(err);
+            throw err;
+          });
       }));
 
       // report any errors
@@ -539,32 +530,6 @@ export default class ClusterModel extends KubeModel {
     };
   }
 
-  async getResourceEndPoint(resource, k8sPaths) {
-    // dynamically get resource endpoint from kebernetes API
-    // ie.https://ec2-54-84-124-218.compute-1.amazonaws.com:8443/kubernetes/
-    if (k8sPaths) {
-      const { apiVersion, kind } = resource;
-      const apiPath = k8sPaths.paths.find((path) => path.match(`/[0-9a-zA-z]*/?${apiVersion}`));
-      if (apiPath) {
-        return (async () => {
-          const k8sResourceList = await this.kubeConnector.get(`${apiPath}`).catch((err) => {
-            logger.error(err);
-            throw err;
-          });
-          const resourceType = k8sResourceList.resources.find((item) => item.kind === kind);
-          const namespace = _.get(resource, 'metadata.namespace');
-          const { name, namespaced } = resourceType;
-          if (namespaced && !namespace) {
-            return null;
-          }
-          const requestPath = `${apiPath}/${namespaced ? `namespaces/${namespace}/` : ''}${name}`;
-          return requestPath;
-        })();
-      }
-    }
-    return undefined;
-  }
-
   async getClusterResources() {
     // Try cluster scope queries, falling back to per-cluster-namespace
     const rbacFallbackQuery = (clusterQuery, namespaceQueryFunction) => (
@@ -749,7 +714,7 @@ export default class ClusterModel extends KubeModel {
       const machinePoolsToDelete = (machinePoolsResponse.items
         && machinePoolsResponse.items
           .filter((item) => _.get(item, 'spec.clusterDeploymentRef.name') === cluster)
-          .map((item) => _.get(item, 'metadata.selfLink'))) || [];
+          .map((item) => `${machinePools}/${_.get(item, 'metadata.name')}`)) || [];
 
       // Create full list of resources to delete
       const resourcesToDelete = [
