@@ -11,6 +11,7 @@
 
 import getApplicationElements, {
   createReplicaChild,
+  createIngressRouteChild,
   createGenericPackageObject,
   addSubscriptionCharts,
   addSubscriptionDeployable,
@@ -23,6 +24,8 @@ import getApplicationElements, {
   removeHelmReleaseName,
   isPrePostHookDeployable,
   createDeployableObject,
+  getLocalClusterElement,
+  createControllerRevisionChild,
 } from './applicationHelper';
 
 describe('createDeployableObject', () => {
@@ -211,6 +214,79 @@ describe('createReplicaChild', () => {
   });
 });
 
+describe('createIngressRouteChild', () => {
+  it('creates a Route object and links it to Ingress', () => {
+    const parentObject = {
+      id: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--ingress--nginx-virtual-placement',
+      uid: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--ingress--nginx-virtual-placement',
+      name: 'nginx-virtual-placement',
+      namespace: 'open-cluster-management',
+      type: 'ingress',
+      specs:
+      {
+        isDesign: false,
+        raw: {
+          kind: 'Ingress',
+          metadata: {
+            name: 'nginx-virtual-placement',
+            namespace: 'open-cluster-management',
+          },
+          spec:
+          {
+            rules: [
+              {
+                host: 'a.b.c',
+              },
+            ],
+          },
+        },
+      },
+    };
+    const template = {
+      kind: 'Ingress',
+      metadata: { name: 'nginx-virtual-placement', namespace: 'open-cluster-management' },
+      spec: {
+        rules: [
+          {
+            host: 'a.b.c',
+          },
+        ],
+      },
+    };
+
+    const result = {
+      id: 'member--member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--route--nginx-virtual-placement',
+      name: 'nginx-virtual-placement',
+      namespace: 'open-cluster-management',
+      specs: {
+        isDesign: false,
+        parent: {
+          parentId: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--ingress--nginx-virtual-placement',
+          parentName: 'nginx-virtual-placement',
+          parentType: 'ingress',
+        },
+        raw: {
+          kind: 'Route',
+          metadata: {
+            name: 'nginx-virtual-placement',
+            namespace: 'open-cluster-management',
+          },
+          spec: {
+            rules: [
+              {
+                host: 'a.b.c',
+              },
+            ],
+          },
+        },
+      },
+      type: 'route',
+      uid: 'member--member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--route--nginx-virtual-placement',
+    };
+    expect(createIngressRouteChild(parentObject, template, [], [])).toEqual(result);
+  });
+});
+
 describe('createGenericPackageObject', () => {
   it('createGenericPackageObject', () => {
     const parentId = 'member--clusters--braveman';
@@ -1310,3 +1386,91 @@ describe('removeHelmReleaseName test resource with only release name as the name
     expect(removeHelmReleaseName('nginx-ingress-66f46', 'nginx-ingress-66f46')).toEqual('nginx-ingress');
   });
 });
+
+describe('removeHelmReleaseName test with package alias not matching name', () => {
+  it('returns package name when resource name matches alias', () => {
+    expect(removeHelmReleaseName('apache-alias-name', 'apache-alias-name-', 'apache', 'apache-alias-name')).toEqual('apache');
+  });
+});
+
+describe('removeHelmReleaseName test with package alias matching name', () => {
+  it('removeHelmReleaseName returns name when resource name does not match alias', () => {
+    expect(removeHelmReleaseName('redis-master', 'my-alias-', 'redis', 'my-alias')).toEqual('redis-master');
+  });
+});
+
+describe('getLocalClusterElement cluster element exists', () => {
+  it('should get the local cluster element', () => {
+    const createdClusterElements = new Set(['member--clusters--cluster1, cluster2, local-cluster']);
+    expect(getLocalClusterElement(createdClusterElements)).toEqual('member--clusters--cluster1, cluster2, local-cluster');
+  });
+});
+
+describe('getLocalClusterElement cluster element does not exists', () => {
+  it('should not get the local cluster element', () => {
+    const createdClusterElements = new Set(['member--clusters--cluster1, cluster2']);
+    expect(getLocalClusterElement(createdClusterElements)).toEqual(undefined);
+  });
+});
+
+describe('createControllerRevisionChild', () => {
+  it('createControllerRevisionChild', () => {
+    const parentObject = {
+      id: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--daemonset--redis-slave',
+      uid: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--daemonset--redis-slave',
+      name: 'redis-slave',
+      namespace: 'open-cluster-management',
+      type: 'daemonset',
+      specs:
+      {
+        isDesign: false,
+        raw: {
+          kind: 'Daemonset',
+          metadata: {
+            name: 'redis-master',
+            namespace: 'open-cluster-management',
+          },
+          spec: { replicas: 3 },
+        },
+      },
+    };
+    const template = {
+      kind: 'Daemonset',
+      metadata: { name: 'redis-slave', namespace: 'open-cluster-management' },
+      spec: {
+        availableReplicas: 2,
+        observedGeneration: 2,
+        readyReplicas: 2,
+        replicas: 2,
+        updatedReplicas: 2,
+      },
+    };
+
+    const result = {
+      id: 'member--member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--controllerrevision--redis-slave',
+      name: 'redis-slave',
+      namespace: 'open-cluster-management',
+      specs: {
+        isDesign: false,
+        parent: {
+          parentId: 'member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--daemonset--redis-slave',
+          parentName: 'redis-slave',
+          parentType: 'daemonset',
+        },
+        raw: {
+          kind: 'controllerrevision',
+          metadata: {
+            name: 'redis-slave',
+            namespace: 'open-cluster-management',
+          },
+          spec: {
+            template: {},
+          },
+        },
+      },
+      type: 'controllerrevision',
+      uid: 'member--member--deployable--member--clusters--possiblereptile, braveman, sharingpenguin, relievedox--controllerrevision--redis-slave',
+    };
+    expect(createControllerRevisionChild(parentObject, template, [], [])).toEqual(result);
+  });
+});
